Unit gChrome;
Interface
Uses Objects,OMemory,OWindows,ODialogs, Strings, gConvert, Global;


Type
{Definition of a chromosome object}
  Pchromosome = ^Tchromosome;
  Tchromosome = Object(TObject)
       Fitness : Real; {fitness of a chromosome}
       Needs_Evaluation : Boolean; {Does a chromosome need evaluating}
       PackedVal : Pchar; {Compressed chromosome}
       Info : PInfo;  {pointer to genetic info}
       BufferLength : Integer;
     Constructor Init(NInfo : PInfo);
     {Creates a new chromosome}
     Procedure UnpackStr(Outstr: Pchar);
     {expands a chromosome to full size (user must provide the buffer for the result)}
     Procedure PackStr(nStr : PChar);
     {Packs an expanded chromosome}
     Procedure UnPackBits(Pos1,Pos2 : integer;Bitval : Pchar);
     {unpacks selected bits of a chromosome into a user defined buffer}
     Procedure PackBits(Pos1,Pos2 : Integer; BitVal:Pchar);
     {Packs selected bits of a chromosome from a user defined buffer}
     Procedure GetBits(Pos1,Pos2 : integer;BitVal: Pchar);
     {gets selected packed bits into a user defined buffer}
     Procedure SetBits(Pos1,Pos2 : Integer; Bits:Pchar);
     {Sets selected Packed bits from a user defined buffer}
     Function GetBit(Pos: integer) : Integer;
     {gets the value of a specific bit}
     Procedure ToggleBit(pos: Integer);
     {Toggles the value o fa specific bit}
     Procedure SetBit(Pos,Bitval : Integer);
     {Sets the value of a specific bit}
     Procedure RandomBits;
     {Sets the bits of a chromosome to a random value}
     Procedure Reset;
     {resets the value of a chromosome}
     Function Comp(nchromosome : Pchromosome) : integer;
     {Compares one chromosome with another returns 1 if nChrom is fitter
     0 if equal -1 if original is fitter}
     Procedure OverWrite(nchromosome : Pchromosome);
     {Overwrites the value of one chromosome with another}
     Destructor Done; Virtual;
     {Destroys an chromosome}
   End;
Implementation

Constructor Tchromosome.Init(NInfo : PInfo);
Begin
  TObject.Init;
  Fitness := 0;
  Needs_Evaluation := True;
  Info := nInfo;
  GetMem(PackedVal,(info^.Buffsize));
  BufferLength := info^.buffsize;
  FillChar(PackedVal^,info^.Buffsize,#0);
End; {init}

Procedure Tchromosome.UnpackStr(OutStr : Pchar);
Var
  f,j,k : Integer;
  n : Byte;
Begin
  For f := 0 to Info^.Full -1 Do 
    Begin
      n := Ord(PackedVal[f]);
      k := f*Charsize;
      For j := 0 to CharSize-1 do
        If (n and Bits[j]) = Bits[j] Then
          OutStr[k+j] := '1'
        Else
          OutStr[K+j] := '0';
    End;    
  If Info^.Slop > 0 Then
    Begin
      n := Ord(PackedVal[info^.Full]);
      k:= Info^.Full * CharSize;
      For j := 0 to Info^.slop -1 Do
       If (n and Bits[j]) = Bits[j] Then
          OutStr[k+j] := '1'
        Else
          OutStr[K+j] := '0';
      OutStr[K+Info^.Slop] := #0;
    End
  Else
    OutStr[K+CharSize] := #0;
End;  {unpack}

Procedure Tchromosome.PackStr(nStr : PChar);
Var
  f,j,k : Integer;
  n : Byte;
Begin
  For f := 0 to info^.Full -1 Do 
    Begin
      n := 0;
      k := f*Charsize;
      For j := 0 to CharSize-1 do
        If nStr[k+j] = '1' Then
          n := n or Bits[j];
      PackedVal[f] := Chr(n);
    End;    
  If Info^.Slop > 0 Then
    Begin
      n := 0;
      k:= info^.Full * CharSize;
      For j := 0 to info^.slop -1 Do
        If nStr[k+j] = '1' Then
          n := n or Bits[j];
      PackedVal[Info^.Full] := Chr(n);
      PackedVal[Info^.Full+1] := #0;
    End
  Else
    PackedVal[info^.Full] := #0;
End; {pack}

Procedure Tchromosome.Reset;
Begin
  Packedval[0] := #0;
  Fitness := 0;
  Needs_Evaluation := True;
End; {Reset}

Procedure Tchromosome.UnPackBits(Pos1,Pos2 : integer;Bitval : Pchar);
Var
  f,n : integer;
Begin
  
  If Pos1 <= Pos2 Then
  Begin
    For f := Pos1 to Pos2 Do 
      BitVal[f-Pos1] := chr(Ord('0')+GetBit(f));    
    BitVal[(Pos2-Pos1)+1] := #0;
  End
  Else
  Begin
    For f := Pos1 to Info^.BitLength-1 Do
      BitVal[f-Pos1] := Chr(Ord('0')+GetBit(f));
    n := Info^.BitLength - Pos1;
    For f := 0 to Pos2 Do
      BitVal[n+f] := Chr(Ord('0')+GetBit(f));
    BitVal[n+Pos2+1] := #0;  
  End;
End;  {unpackbits}

Procedure Tchromosome.PackBits(Pos1,Pos2 : Integer; BitVal:Pchar);
Begin
  Abstract;
End; {PackBits}

Function Tchromosome.GetBit(Pos: integer) : Integer;
Var
  Tmp : Char;
Begin
  Tmp := PackedVal[(Pos div Charsize)];
  tmp := Char(Integer(Tmp) shr (Charsize-(pos Mod Charsize)-1));
  GetBit := integer(tmp) ANd Bits[7];
End; {GetBit}

Procedure Tchromosome.ToggleBit(pos: Integer);
 Var
  val : integer;
Begin
  val :=  Not(GetBit(Pos)) And Bits[7];

  SetBit(Pos,(val));
End; {ToggleBit}

Procedure Tchromosome.RandomBits;
Var
  f : Integer;
Begin
  For f := 0 to Info^.BitLength-1 do
    self.SetBit(f,Random(2));
End; {Random Bits}

Procedure Tchromosome.SetBit(Pos,BitVal : Integer);
Begin
  If BitVal = 0 Then
    PackedVal[(Pos div Charsize)] := Char(Integer(PackedVal[(Pos div Charsize) ]) And
    (Not Bits[Pos Mod Charsize]))
  Else
    PackedVal[(Pos div Charsize)] := Char(Integer(PackedVal[(Pos div Charsize) ]) Or
    Bits[Pos Mod Charsize]);
End; {setBit}

Function Tchromosome.Comp(nchromosome : Pchromosome) : integer;
Begin
    Comp := -1;
  If nchromosome^.Fitness > Fitness Then
    Comp := 1;
  If nchromosome^.Fitness = Fitness Then
    Comp := 0;
End; {comp}

Procedure Tchromosome.OverWrite(nchromosome : Pchromosome);
var
  f: integer;
  n: char;
Begin
  Fitness := nchromosome^.fitness;
  Needs_Evaluation := nchromosome^.Needs_Evaluation;
  {Move(nchromosome^.PackedVal,PackedVal,info^.BuffSize-1);}
  for f := 0 to info^.Buffsize-1 do
  Begin
    n := nchromosome^.PackedVal[f];
    PackedVal[f] := n;
  End;
End;  {OverWrite}

Destructor Tchromosome.Done;
Begin
  FreeMem(PackedVal,bufferLength);
  TObject.Done;
End;   {done}

Procedure Tchromosome.GetBits(Pos1,Pos2 : integer;Bitval : Pchar);
Begin
  Abstract;
End; {getbits}

Procedure Tchromosome.SetBits(Pos1,Pos2 : Integer; Bits:Pchar);
Begin
  Abstract;
End;  {setbits}
End. 