Unit gCross;
Interface
Uses Global, Strings, gChrome,Objects,OMemory,OWindows,ODialogs,winprocs,winTypes,gpool1;
Const
  CrossCount = 3;

Type
  TCrossFunc = Procedure(Info: PInfo;P1,P2,C1,C2: Pchromosome);


Type
  PCrossType = ^TCrossType;
  TCrossType = Object(TObject)
     FuncIndx  : Integer; {current index in Cross Table}
     CrossFunc : TCrossFunc; {Currently selected crossover procedure}
     Constructor Init(Name : Pchar);
     Function GetCrossCount : Integer;
     Procedure SetFunc(Name : Pchar);
     Function GetFuncName : Pchar;
     Function GetNameAt(Pos:integer) : Pchar;
     Procedure CrossOver(Info:Pinfo; P1,P2,C1,C2 : Pchromosome);
     Destructor Done; Virtual;
   End;

  PRecombine = ^TRecombine;
  TRecombine = Object(TCrossType)
     Constructor Init(CrossName : Pchar);
     Procedure Cross(Pool1, Pool2: PPool; Info: Pinfo);
     Destructor Done; Virtual;
  End;

Procedure Uniform(Info:Pinfo;P1,P2,C1,C2 : Pchromosome);
Procedure Cross1(Info:Pinfo;P1,P2,C1,C2: Pchromosome);
Procedure Cross2(Info:Pinfo;P1,P2,C1,C2: Pchromosome);
Procedure Cross2R(Info:Pinfo;P1,P2,C1,C2: Pchromosome);
Procedure CrossN(Info:Pinfo;P1,P2,C1,C2: Pchromosome);
Procedure CrossD(Info:Pinfo;P1,P2,C1,C2: Pchromosome);

Implementation
Type
  CrossRec = Record
      Func : TCrossFunc;
      Description : Pchar;
  End;
Const
  CrossTable : Array[0..CrossCount-1] of CrossRec = (
     {(Func: Uniform; Description : 'Uniform CrossOver'),}
     (Func: Cross1;  Description : 'One point CrossOver'),
     (Func: Cross2;  Description : 'Two point Crossover'),
     {(Func: CrossN;  Description : 'N point Crossover'),
     (Func: CrossD;  Description : 'Discrete Crossover'),   }
     (Func: Cross2R; Description : 'Two point Ring Crossover')
      );

     
Constructor TCrossType.Init(Name : Pchar);
Begin 
  TObject.Init;
  FuncIndx := 0;
  If Name = nil Then
    CrossFunc := CrossTable[FuncIndx].Func
  Else
    SetFunc(Name);
End;

Function TCrossType.GetCrossCount : Integer;
Begin
  GetCrossCount := CrossCount;
End;

Procedure TCrossType.SetFunc(Name : Pchar);
Var 
  f: Integer;
  found : Boolean;
Begin
  For f := 0 to CrossCount-1 Do
    If StrComp(Name,CrossTable[f].Description) = 0 Then
    Begin
      CrossFunc := CrossTable[f].func;
      FuncIndx := f;
      found := True;
    End;
  If Not Found Then
    CrossFunc := CrossTable[FuncIndx].func;
End;

Function TCrossType.GetFuncName : Pchar;
Begin
   GetFuncName := CrossTable[FuncIndx].Description;
End;

Function TCrossType.GetNameAt(Pos:integer) : Pchar;
Begin
  If (Pos < 0) Or (Pos >= CrossCount) Then 
    GetNameAt := nil
  Else
    GetNameAt := CrossTable[pos].Description;
End;

Procedure TCrossType.CrossOver(Info: PINFO; P1,P2,C1,C2 : Pchromosome);
Begin
  CrossFunc(info, p1,p2,c1,c2);
End;

Destructor TCrossType.Done; 
Begin
  TObject.Done;
End;

Constructor TRecombine.Init(CrossName : Pchar);
Begin
  TCrossType.init(CrossName);
End;

Procedure TRecombine.Cross(Pool1, Pool2: PPool; Info: Pinfo);
Var
  P1,P2,C1,C2: Pchromosome;
  f,g : Integer;
  last : integer;
   nmsg :Tmsg;
Begin
  If info^.Running Then
  Begin
    Last := (Round(Pool1^.count * Info^.CrossRate) div 2)-1;
    For f := 0 to Last do
    Begin
      g := f*2;
      P1 := Pool1^.at(g);
      P2 := Pool1^.at(g+1);
      C1 := Pool2^.at(g);
      C2 := Pool2^.at(g+1);
      Crossover(Info,P1,P2,C1,C2);
      While peekMessage(nMsg,0,0,0,pm_remove) Do
        If nMsg.Message = wm_quit Then Halt
      Else if not Application^.ProcessAppMsg(nMsg) Then
      Begin
        TranslateMessage(nmsg);
        DispatchMessage(nmsg);
      End;

  End;
  For f := (Last*2)+2 to Pool1^.count - 1 Do
   Begin
    Pchromosome(Pool2^.at(f))^.Overwrite(Pchromosome(POOL1^.at(f)));
   end;
{  If (Round(Pool1^.count * Info^.CrossRate) mod 2) <> 0 Then
    Pool2^.atPut(Round(Pool1^.count * Info^.CrossRate),
     Pool1^.at(Round(Pool1^.count * Info^.CrossRate)));
}
  End;
End;

Destructor TRecombine.Done;
Begin
  TCrossType.Done;
End;

Procedure Cross1( Info:PInfo;P1,P2,C1,C2 : Pchromosome);
Var
  pos,f: Integer;
  Bytes,Bits : Integer;
  tmp1,tmp2,tmp3,tmp4 : Byte;
  Same : Boolean;
Begin
  Same := True;
  Pos := Random(Info^.Bitlength);
  Bytes := Pos div Charsize;
  Bits := Pos mod Charsize;
  Move(P1^.PackedVal^,C1^.PackedVal^,info^.BuffSize-1);
  Move(P2^.PackedVal^,C2^.PackedVal^,info^.BuffSize-1);
{  If StrLcomp(P1^.PackedVal,P2^.PackedVal,Bytes) <> 0 Then
    Same := False
  Else }
  f := 0;
  While (f < bytes) and Same do
  begin
   Same := (P1^.PackedVal[f] = P2^.PackedVal[f]);
   f := F+1;
  end;  

  If same and (Bits <> 0) Then
  Begin
    If (Byte(P1^.PackedVal[Bytes]) And BMask2[Bits]) <>
       (Byte(P2^.PackedVal[Bytes]) And BMask2[Bits]) Then
          Same := False;
  end  ;
  If(Pos = 0) Or (Pos >= info^.Bitlength) THen
    Same := TRue;
  If Not Same Then
  Begin
    C1^.Needs_Evaluation := True;
    C2^.Needs_Evaluation := True;
    Move(P2^.PackedVal[Bytes+1],C1^.PackedVal[bytes+1],(Info^.BuffSize-Bytes)-1);
    Move(P1^.PackedVal[Bytes+1],C2^.PackedVal[bytes+1],(Info^.BuffSize-Bytes)-1);
    If Bits = 0 Then
    Begin
      C1^.PackedVal[Bytes] := P2^.PackedVal[Bytes];
      C2^.PackedVal[Bytes] := P1^.PackedVal[Bytes];    
    End
    Else
    Begin
      Tmp1 := Byte(P1^.PackedVal[Bytes]) And BMask1[Bits];
      Tmp2 := Byte(P1^.PackedVal[Bytes]) And BMask2[Bits];
      Tmp3 := Byte(P2^.PackedVal[Bytes]) And BMask1[Bits];
      Tmp4 := Byte(P2^.PackedVal[Bytes]) And BMask2[Bits];
      C1^.PackedVal[Bytes] := Char(Tmp2 or Tmp3);
      C2^.PackedVal[Bytes] := Char(Tmp4 or Tmp1);
    End;
  End
  Else
  Begin
    C1^.Needs_Evaluation := False;
    C2^.Needs_Evaluation := False;
  End;
End;

Procedure Cross2( Info:PInfo;P1,P2,C1,C2 : Pchromosome);
Var
  pos,pos2,nbytes,n ,f: Integer;
  Bytes,Bits : Integer;
  Bytes1,Bits1 : Integer;
  tmp1,tmp2,tmp3,tmp4 : Byte;
  Same : Boolean;
Begin
  Same := False;
  Pos := Random(info^.Bitlength);
  Pos2 := Random(info^.BitLength);
  If Pos > Pos2 then
  Begin
    n := Pos2;
    Pos2 := Pos;
    Pos := n;
  End;
  {n := Pos2-Pos;}
  Bytes := Pos div Charsize;
  Bits := Pos mod Charsize;
  Bytes1 := {(Pos+n)}Pos2 div Charsize;
  Bits1 := {(Pos+n)}Pos2 mod Charsize;
  If Bytes <> Bytes1 Then
    nbytes := (Bytes1-Bytes)
  Else
    nBytes := 0;
  Move(P1^.PackedVal^,C1^.PackedVal^,info^.BuffSize-1);
  Move(P2^.PackedVal^,C2^.PackedVal^,info^.BuffSize-1);

  If StrLcomp(P1^.PackedVal,P2^.PackedVal,Bytes) <> 0 Then
    Same := False
  Else
  If Bits <> 0 Then
  Begin
    If (Byte(P1^.PackedVal[Bytes]) And BMask2[Bits]) =
       (Byte(P2^.PackedVal[Bytes]) And BMask2[Bits]) Then
          Same := True;
  end  ;
  If(Pos = 0) Or (Pos > info^.Bitlength) THen
    Same := TRue;


  If Same Then
  Begin
  If StrLcomp(addr(P1^.PackedVal[bytes1]),
              addr(P2^.PackedVal[bytes1]),(info^.BuffSize-Bytes)-1) <> 0 Then
    Same := False
  Else
  If Bits <> 0 Then
  Begin
    If (Byte(P1^.PackedVal[Bytes1]) And BMask1[Bits]) =
       (Byte(P2^.PackedVal[Bytes1]) And BMask1[Bits]) Then
          Same := True;
  end  ;
  If(Pos2 = 0) Or (Pos2 > info^.Bitlength) THen
    Same := TRue;  
  End;

  If Not Same Then
  Begin
    C1^.Needs_Evaluation := True;
    C2^.Needs_Evaluation := True;
    Move(P2^.PackedVal[Bytes+1],C1^.PackedVal[bytes+1],nbytes);
    Move(P1^.PackedVal[Bytes+1],C2^.PackedVal[bytes+1],nbytes);

    Tmp1 := Byte(P1^.PackedVal[Bytes]) And BMask1[Bits];
    Tmp2 := Byte(P1^.PackedVal[Bytes]) And BMask2[Bits];
    Tmp3 := Byte(P2^.PackedVal[Bytes]) And BMask1[Bits];
    Tmp4 := Byte(P2^.PackedVal[Bytes]) And BMask2[Bits];
    C1^.PackedVal[Bytes] := Char(Tmp2 or Tmp3);
    C2^.PackedVal[Bytes] := Char(Tmp4 or Tmp1);

    Tmp1 := Byte(P1^.PackedVal[Bytes1]) And BMask1[Bits1];
    Tmp2 := Byte(C1^.PackedVal[Bytes1]) And BMask2[Bits1];
    Tmp3 := Byte(P2^.PackedVal[Bytes1]) And BMask1[Bits1];
    Tmp4 := Byte(C2^.PackedVal[Bytes1]) And BMask2[Bits1];
    C1^.PackedVal[Bytes1] := Char(Tmp2 or Tmp1);
    C2^.PackedVal[Bytes1] := Char(Tmp4 or Tmp3);
  End;
End;

Procedure Cross2R( Info:PInfo;P1,P2,C1,C2 : Pchromosome);
Var
  pos,nbytes,n ,f: Integer;
  Bytes,Bits : Integer;
  Bytes1,Bits1,Bytes2 : Integer;
  tmp1,tmp2,tmp3,tmp4 : Byte;
  Same : Boolean;
Begin
  Same := True;
  Pos := Random(info^.Bitlength);
  n := Random(info^.BitLength);
  Bytes := Pos div Charsize;
  Bits := Pos mod Charsize;
  Bytes1 := (Pos+n) div Charsize;
  Bits1 := (Pos+n) mod Charsize;
  If Bytes <> Bytes1 Then
    nbytes := (Bytes1-Bytes)
  Else
    nBytes := 0;
  c1^.Overwrite(P1);
  c2^.Overwrite(p2);
 { Move(P1^.PackedVal^,C1^.PackedVal^,info^.BuffSize-1);
  Move(P2^.PackedVal^,C2^.PackedVal^,info^.BuffSize-1);
  }
 {}
 If (Pos + n) > Info^.Bitlength Then
  Begin
    Bits1 := (( Pos+n )-Info^.Bitlength) Mod Charsize;
    Bytes2 := ((Pos+n) mod info^.bitlength) div Charsize;

    f := Bytes2;
    While (f < bytes) and Same  do
    begin
     Same := (P1^.PackedVal[f] = P2^.PackedVal[f]);
     inc(f);
    end;
    If  Byte(P1^.PackedVal[Bytes]) And BMask2[Bits] <>
        Byte(P2^.PackedVal[Bytes]) And BMask2[Bits] Then
        Same := False;                      
    If  Byte(P1^.PackedVal[Bytes2]) And BMask1[Bits1] <>
        Byte(P2^.PackedVal[Bytes2]) And BMask1[Bits1] Then
        Same := False;
  end
  Else
  Begin
    f :=0;
    While (f < bytes) and Same  do
    begin
     Same := (P1^.PackedVal[f] = P2^.PackedVal[f]);
     inc(f);
    end;
    If  Byte(P1^.PackedVal[Bytes]) And BMask2[Bits] <>
        Byte(P2^.PackedVal[Bytes]) And BMask2[Bits] Then
        Same := False;
    If  Byte(P1^.PackedVal[Bytes1]) And BMask1[Bits1] <>
        Byte(P2^.PackedVal[Bytes1]) And BMask1[Bits1] Then
        Same := False;
    f :=bytes1+1;
    While (f < info^.Buffsize-1) and Same  do
    begin
     Same := (P1^.PackedVal[f] = P2^.PackedVal[f]);
     inc(f);
    end;
  end;

  Begin
  If (Pos + n) > Info^.Bitlength Then
  Begin
    Bits1 := (( Pos+n )-Info^.Bitlength) Mod Charsize;
    Bytes2 := ((Pos+n) mod info^.bitlength) div Charsize;
    Move(P2^.PackedVal[Bytes+1],C1^.PackedVal[bytes+1],(Info^.BuffSize-Bytes)-1);
    Move(P1^.PackedVal[Bytes+1],C2^.PackedVal[bytes+1],(Info^.BuffSize-Bytes)-1);
    Move(P2^.PackedVal[0],C1^.PackedVal[0],Bytes2+1);
    Move(P1^.PackedVal[0],C2^.PackedVal[0],Bytes2+1);
  End
  Else
  Begin
    Move(P2^.PackedVal[Bytes+1],C1^.PackedVal[bytes+1],nbytes);
    Move(P1^.PackedVal[Bytes+1],C2^.PackedVal[bytes+1],nbytes);
  End;

  Tmp1 := Byte(P1^.PackedVal[Bytes]) And BMask1[Bits];
  Tmp2 := Byte(P1^.PackedVal[Bytes]) And BMask2[Bits];
  Tmp3 := Byte(P2^.PackedVal[Bytes]) And BMask1[Bits];
  Tmp4 := Byte(P2^.PackedVal[Bytes]) And BMask2[Bits];
  C1^.PackedVal[Bytes] := Char(Tmp2 or Tmp3);
  C2^.PackedVal[Bytes] := Char(Tmp4 or Tmp1);

  If (Pos + n) > Info^.Bitlength Then
  Begin
    Tmp1 := Byte(P1^.PackedVal[Bytes2]) And BMask1[Bits1];
    Tmp2 := Byte(C1^.PackedVal[Bytes2]) And BMask2[Bits1];
    Tmp3 := Byte(P2^.PackedVal[Bytes2]) And BMask1[Bits1];
    Tmp4 := Byte(C2^.PackedVal[Bytes2]) And BMask2[Bits1];
    C1^.PackedVal[Bytes2] := Char(Tmp2 or Tmp1);
    C2^.PackedVal[Bytes2] := Char(Tmp4 or Tmp3);
  End
  Else
  Begin
    Tmp1 := Byte(P1^.PackedVal[Bytes1]) And BMask1[Bits1];
    Tmp2 := Byte(C1^.PackedVal[Bytes1]) And BMask2[Bits1];
    Tmp3 := Byte(P2^.PackedVal[Bytes1]) And BMask1[Bits1];
    Tmp4 := Byte(C2^.PackedVal[Bytes1]) And BMask2[Bits1];
    C1^.PackedVal[Bytes1] := Char(Tmp2 or Tmp1);
    C2^.PackedVal[Bytes1] := Char(Tmp4 or Tmp3);
  End;
  end;

  If Not Same Then
  begin
  C1^.Needs_Evaluation := True;
  C2^.Needs_Evaluation := True;

  End
  Else
  Begin
    C1^.Needs_Evaluation := false;
    C2^.Needs_Evaluation := false;
  End;
 { P1^.UnpackStr(buf);
  c1^.UnpackStr(buf1);
  Writeln(fp,buf,' ',buf1,' ',c1^.Needs_Evaluation);
  P2^.UnpackStr(buf);
  c2^.UnpackStr(buf1);
  Writeln(fp,buf,' ',buf1,' ',c2^.Needs_Evaluation);
  }
End;


Procedure Uniform( Info:PInfo;P1,P2,C1,C2 : Pchromosome);
Begin
End;

Procedure CrossN(Info:Pinfo;P1,P2,C1,C2: Pchromosome);
Var
  T1,T2 : Pchromosome;
  f : integer;
Begin
 { For f := 0 to Info^.NumPoints
  }
End;

Procedure CrossD(Info:Pinfo;P1,P2,C1,C2: Pchromosome);
Begin
End;

End.
