Unit gGene;
{

    file:	Gene.pas
  
    purpose:	Contains object definition of a gene for conversion of
                chromosones to datatypes

    modified:	Currently only supports reals
}
Interface
Uses gconvert,global,Objects,OMemory,OWindows,ODialogs, strings;


Const

{Constants for defining data types}
 GShort   = 1;  {-128..127  signed 8-bit}
 GInt     = 2;  {-32768..32767 signed 16-bit}
 GLong    = 3;  {-2147483648..2147483647 signed 32-bit}
 GByte    = 4;  {0..255 unsigned 8-bit}
 GWord    = 5;  {0..65535 unsigned 16-bit}
 GReal    = 6;   
 GSingle  = 7;
 GDouble  = 8;
 GExt     = 9;
 GComp    = 10;
 GPchar   = 11;
 GBoolean = 12;

 NumTypes = 12; {Number of defined datatypes}

 GLength : array[1..NumTypes] of byte = (1,2,4,1,2,6,4,8,10,8,4,1);
 {This array contains the number of bytes which are required to
  represent a datatype}



Type

  TEvalFunc = Function(pVect : Pointer; nLength : Integer):Real;
  TPEvalFunc = Function(pVect : Pointer; nLength : Integer):Real;
  {Prototype function definitions for external evaluation functions}



{Definition of a gene object}
PGene = ^TGene;
TGene = Object(TObject)
    GeneType   : Integer; {Gene identifier}
    BitCount   : Word;    {Number of bits required in bit string}
    ByteCount  : Integer;
    MaxRes     : LongInt; {Maximum resolution of a datatype}
    MinRes     : LongInt; {Minimum resolution of a datatype}
    CurrentRes : LongInt; {Current resolution of a datatype}
    MinRVal    : Real;    {Minimum value of a real}
    MaxRVal    : Real;    {Maximum value of a real}
    MinIVal    : Longint; {Minimum value of an integer}
    MaxIval    : Longint; {Maximum value of an integer}
    Offset     : Real;    {current offset for DPE}
    Incr       : Real;    {Increment for each value}
    BitBuff    : Pointer; {Pointer to buffer holding the value}
    Info       : Pinfo;   {Pointer to algorithm info}
    Default    : Boolean;
    tmpBuff    : Pointer;
    Varstr    : Pointer;
    VarLength : Integer;
    Constructor Init(nInfo : Pinfo;nBitCount:Word;nByteCount : integer;
                     nDefault : Boolean);
    {constructs a new Gene}
    Function GetBitCount : Integer;   Virtual;
    {Returns the number of chromosone bits required to represent a datatype}
    Function GetByteCount : Integer;  Virtual;
    {Returns the number of bytes required to represent the final datatype}
    Function GetVal(BitStr: Pchar; Buffsize: Word): Pointer; Virtual;
    {Returns a pointer to a value which represents the value of a gene} 
   Procedure GetValue(BitStr: Pchar;Result:Pchar;ResultLength : integer); Virtual;
    Function GetType:Pchar; Virtual;
    {Retunrns a pointer to a string which describes the gene type}
    Destructor Done; Virtual;
    {Destroys a gene}
  End;


PGShort = ^TGShort;
TGShort = Object(TGene)
   Constructor init( NInfo : Pinfo);
   Function GetBitCount : Integer; Virtual;
   Function GetByteCount : Integer; Virtual;
   Function GetVal(BitStr: Pchar; Buffsize: Word):Pointer; Virtual;
   Procedure GetValue(BitStr: Pchar;Result:Pchar;ResultLength : integer); Virtual;
   Function GetType:Pchar; Virtual;
   Destructor Done; virtual;
end;

PGPChar = ^TGPChar;
TGPChar = Object(TGene)
   Constructor init( NInfo : Pinfo; nLength:integer);
   Function GetBitCount : Integer; Virtual;
   Function GetByteCount : Integer; Virtual;
   Function GetVal(BitStr: Pchar; Buffsize: Word):Pointer; Virtual;
   Procedure GetValue(BitStr: Pchar;Result:Pchar;ResultLength : integer); Virtual;
   Function GetType:Pchar; Virtual;
   Destructor Done; virtual;
end;

PGReal = ^TGReal;
TGReal = Object(TGene)
   Constructor init(nDefault:Boolean;Min,Max:Real;Resolution:Longint;
                    NInfo : Pinfo);
   Function GetBitCount : Integer; Virtual;
   Function GetByteCount : Integer; Virtual;
   Function GetVal(BitStr: Pchar; Buffsize: Word):Pointer; Virtual;
   Procedure GetValue(BitStr: Pchar;Result:Pchar;ResultLength : integer); Virtual;
   Function GetType:Pchar; Virtual;
   Destructor Done; virtual;
end;



{A GeneList is a list of genes which represent an entire chromosone}
PGeneList = ^TGeneList;
TGeneList = Object(TCollection)
     ByteBuff    : Pointer; {pointer to a buffer containing all the gene
                            values}
     BuffLength : Integer;  {size of the buffer}
     Tmp        : Pointer;  {temp pointer}
     Info       : Pinfo;    {Pointer to algorithm info}
   Constructor init(nInfo : Pinfo);
   {constructs a new gene list}
   Function GetBitCount : Integer;
   {gets the total number of bytes required to represent the gene ie the
   Chromosone length}
   Function GetByteCount : Integer;
   {Gets the total number of bytes require to hold the gene datatypes}
   Function GetVec(PBits : Pchar) : Pointer;
   {Converts a chromosone(bitString)to a list of parameters}
   Procedure Add(gene : Pgene);
   {Adds another gene to a gene list}
   Procedure Empty;
   {Empty a genelist;}
   Destructor Done; virtual;
   {Destroys a gene list}
  end;

Implementation


Constructor Tgene.Init(nInfo : Pinfo;nBitCount:Word;NByteCount:integer;
                       nDefault : Boolean);
Begin
  Tobject.Init;
  Info := nInfo;
  Default := nDefault;
  BitCount := nBitCount;
  ByteCount := nByteCount;
  VarStr := nil;
  If MaxAvail <= ByteCount Then
      {MEMERROR('Out of memory to assign gene')};
    Getmem(BitBuff,ByteCount);
    Getmem(TmpBuff,ByteCount+1);
End; {Init}

Destructor TGene.Done;
Begin
  If VarStr <> nil Then
    StrDispose(VarStr);
  If BitBuff <> nil Then
     FreeMem(BitBuff,ByteCount);
  If TmpBuff <> nil Then
     FreeMem(TmpBuff,ByteCount+1);
  TObject.Done;
End; {Done}

Function Tgene.GetBitCount : Integer;
Begin
  GetBitCount := BitCount;
End; {getbitCount}

Function Tgene.GetByteCount;
Begin
   GetByteCount := ByteCount;
end; {GetByteCount}

Function TGene.GetVal(BitStr: Pchar; Buffsize: Word): Pointer;
Begin
  Abstract;
End; {GetVal}

Procedure Tgene.GetValue(BitStr: Pchar;Result:Pchar;ResultLength : integer);
Begin
  Abstract;
end;

Function TGene.GetType:Pchar;
Begin
  Abstract;
End;



Constructor TGeneList.init(nInfo : Pinfo);
Begin
  Info := ninfo;
  BuffLength := 0;
  TCollection.init(10,5);
  ByteBuff := nil;
End; {init}

Procedure TgeneList.Add(gene : Pgene);
Begin
  Insert(gene);
  If ByteBuff <> nil Then
    FreeMem(ByteBuff,BuffLength);
  BuffLength := Bufflength + Gene^.GetByteCount;
  If MaxAvail <= BuffLength Then
      {MEMERROR('Out of memory to assign genelist')};
  Getmem(ByteBuff,BuffLength);
End;  {Add}


Procedure TGeneList.Empty;
Begin
 If BuffLength <> 0 Then
 Begin
   FreeMem(ByteBuff,BuffLength);
   BuffLength :=0;
   ByteBuff := nil;
 End;
 If Count > 0 Then
   DeleteAll;
End;
  
Function TGeneList.GetBitCount : Integer;
Var
  f,s : integer;
Begin
  s := 0;
  For f := 0 to count-1 do
    s := s + Pgene(at(f))^.GetBitCount;
  GetBitCount := s;
End;  {GetBitCount}

Function TGeneList.GetByteCount : Integer;
Var
  f,s : integer;
Begin
  s := 0;
  For f := 0 to Count-1 do
    s := s+ PGene(at(f))^.GetByteCount;
  GetByteCount := s;
End; {GetByteCount}

Function TGeneList.GetVec(PBits : Pchar) : Pointer;
Var
  f,s,d,pos : integer;
  Temp : Pgene;
  TempChar : Pchar;
    P : pOINTER;
Begin
  tmp := ByteBuff;
  For f := 0 to count-1 do
  Begin
    Temp := Pgene(at(f));
    Move(Temp^.GetVal(PBits,StrLen(Pbits))^,
         ByteBuff^,
         temp^.GetByteCount);
    inc(tPtr(ByteBuff).Ofs,Temp^.GetByteCount);
    Pbits := PBits + Temp^.GetBitCount;
  End;
  ByteBuff := Tmp;
  GetVec := Tmp;
End; {GetVec}


Destructor TGeneList.Done;
Begin
  TCollection.done;
end;  {done}



Constructor TGShort.init( NInfo : Pinfo);
Begin
  BitCount := 8;
  ByteCount := 1;
  Tgene.init(Ninfo,BitCount,ByteCount,False);
End;

Function TGShort.GetBitCount : Integer; 
Begin
  GetBitCount := BitCount;
End;

Function TGShort.GetByteCount : Integer; 
Begin
  GetByteCount := ByteCount;
End;

Function TGShort.GetVal(BitStr: Pchar; Buffsize: Word):Pointer; 
Var
  n : Longint;
Begin
  If BuffSize < BitCount Then
  Begin
    GetVal := nil;
    Exit;
  End;
    PackChar(BitStr,Pchar(TmpBuff),BitCount);
    ShortInt(BitBuff^) := Shortint(TmpBuff^);
    GetVal := BitBuff;
End; {GetVal}
 
Procedure TGSHort.GetValue(BitStr: Pchar;Result:Pchar;ResultLength : integer);
Var
  n : Longint;
  d : Shortint;
  s : String;
Begin
  PackChar(BitStr,Pchar(TmpBuff),BitCount);
  d := Shortint(TmpBuff^);
  Str(d,s);
  StrPcopy(Result,s);
End; {GetVal}

Function TGShort.GetType:Pchar;
Begin
  GetType := 'Short int';
End;

Destructor TGShort.Done;
Begin
  TGene.Done;
end;

Constructor TGReal.init(nDefault:Boolean;Min,Max:Real; Resolution : Longint;
                           NInfo : Pinfo);
Begin
  ByteCount := 6;
  VarLength := 0;
  MinRes := Resolution;
  If nDefault Then
  Begin
    MinRVal := 2.9E-39;
    MaxRVal := 1.7E38;
    BitCount := ByteCount * Charsize;
  End
  Else
  Begin
    MinRVal := Min;
    MaxRVal := Max;
    Incr := (MaxRVal-MinRval)/(MinREs-1);
    BitCount := GetBitLength(MinRes);
  End;
  Tgene.init(Ninfo,BitCount,ByteCount,nDefault);
End;

Function TGReal.GetBitCount : Integer; 
Begin
  GetBitCount := BitCount;
End;

Function TGReal.GetByteCount : Integer; 
Begin
  GetByteCount := ByteCount;
End;


Procedure TGReal.GetValue(BitStr: Pchar;Result:Pchar;ResultLength : integer);
Var
  n : Longint;
  R : Real;
  S : String;
Begin
  If Default Then
  Begin
    PackChar(BitStr,Pchar(TmpBuff),BitCount);
    Real(BitBuff^) := Real(TmpBuff^);
  End
  else
  Begin
    n := StrToInt(BitStr,BitCount);
    Real(BitBuff^) := MinRval + n * Incr;
  End;
  R := Real(BitBuff^);
  Str(R:8:5,s);
{    Freemem(Varstr,VarLength);
  VarLength := Length(s)+1;
  GetMem(varStr,VarLength);
 } StrPcopy(Result,s);
  {GetValue := VarStr;
    }
 End; {GetVal}

Function TGReal.GetVal(BitStr: Pchar; Buffsize: Word):Pointer;
Var
  n : Longint;

Begin
  If BuffSize < BitCount Then
  Begin
    GetVal := nil;
    Exit;
  End;
  If Default Then
  Begin
    PackChar(BitStr,Pchar(TmpBuff),BitCount);
    Real(BitBuff^) := Real(TmpBuff^);
  End
  else
  Begin
    n := StrToInt(BitStr,BitCount);
    Real(BitBuff^) := MinRval + n * Incr;
  End;
  GetVal := BitBuff;
 End; {GetVal}

Function TGReal.GetType:Pchar;
Var
  R : Real;
  S,v : String;
Begin
  If Default Then
    GetType := 'Real'
  Else
  Begin
     S := 'Real ';
     Str(MinRVal:6:3,v);
     S := S+V +' to ';
     Str(MaxRVal:6:3,v);
     S := S+V;
  If VarStr <> nil Then
    StrDispose(Varstr);
  Getmem(varStr,Length(s)+1);
  StrPcopy(VarStr,s);
  GetType := VarStr;
  End;
End;


Destructor TGReal.Done;
Begin
  TGene.Done;
end;


Constructor TGPChar.init( NInfo : Pinfo; nLength : Integer);
Begin
  BitCount := nLength;
  ByteCount := (nLength div CharSize) + 2;
  Tgene.init(Ninfo,BitCount,ByteCount,False);
End;

Function TGPChar.GetBitCount : Integer;
Begin
  GetBitCount := BitCount;
End;

Function TGPChar.GetByteCount : Integer;
Begin
  GetByteCount := ByteCount;
End;

Procedure TGPChar.GetValue(BitStr: Pchar;Result:Pchar;ResultLength : integer);
Var
  n : Longint;
Begin
    StrLCopy(Pchar(BitBuff),BitStr,BitCount);
    Pchar(BitBuff)[BitCount] := #0;
    StrLcopy(Result,BitBuff,ResultLength);
End; {GetVal}

Function TGPChar.GetVal(BitStr: Pchar; Buffsize: Word):Pointer;
Var
  n : Longint;
Begin
  If BuffSize < BitCount Then
  Begin
    GetVal := nil;
    Exit;
  End;
    StrLCopy(Pchar(BitBuff),BitStr,BitCount);
    Pchar(BitBuff)[BitCount] := #0;
    GetVal := BitBuff;
End; {GetVal}

Function TGPChar.GetType : Pchar;
Begin
  GetType := 'PChar'
End;

Destructor TGPChar.Done;
Begin
  TGene.Done;
end;

end.