Unit GGraph1;
Interface
 uses WinTypes,WinProcs, Objects, OWindows, Pen, strings;
 Const
   AxisPt = 3; {Length of axis point line}

 Type
  AxisRec = Record
         TitleOn : Boolean;    {DisplayTitle}
         MinMaxOn : Boolean;  {display min and max}
         NumOn : Boolean;    {display all numbers}
         Min,Max : Real;
         Name : Array[0..100] of char;
         Scale : Real;
     End;

  PAxis = ^TAxis;
  TAxis = Object(TObject)
         AXPen : PPen;
         TitleOn : Boolean;    {DisplayTitle}
         MinMaxOn : Boolean;  {display min and max}
         NumOn : Boolean;    {display all numbers}
         AutoScale : Boolean;
     Constructor init(nname:Pchar);
     Procedure SetRange(DC:HDC;nMin,nMax : Real); virtual; {Set min and max values}
     Procedure SetScale(ndif : Real);  {set scale points}
     Procedure SetRectSize(DC:HDC;nR:Trect); Virtual; {SetRect to hold axis}
     Procedure SetName(nName : Pchar);
     Function GetMinW(DC:HDC):integer; Virtual; {Get max hheight of axis}
     Function GetMinH(DC:HDC):integer; Virtual; {Get min height of axis}
     Procedure Draw(DC:HDC); virtual;
     Destructor Done; Virtual;
     Private
       R:Trect;
       TextOn : Boolean;
       Min,Max : Real;
       Name : Array[0..100] of char;
       Dif : Real;
       Scale : Real;
       OffsetSrc: Real;
       OffsetDest: Integer;
       Function Translate(src: Real) : integer;
    end;

  PHorAxis = ^THorAxis;
  THorAxis = Object(TAxis)
     Procedure SetRange(DC:HDC;nMin,nMax : Real);  virtual;
     Procedure SetRectSize(DC:HDC;nR:Trect); Virtual;
     Function GetMinh(DC:HDC) : Integer; virtual;

     Procedure Draw(DC:HDC); Virtual;
  end;

  PVerAxis = ^TVerAxis;
  TVerAxis = Object(TAxis)
     Procedure SetRange(DC:HDC;nMin,nMax : Real);  virtual;
     Procedure SetRectSize(DC:HDC;nR:Trect); Virtual;
     Function GetMinW(DC:HDC) : Integer; virtual;
     Function GetMinh(DC:HDC):integer; Virtual;
     Procedure Draw(DC:HDC); Virtual;
  end;

Implementation



Constructor TAxis.init(nname:Pchar);
Begin
  TObject.init;
  TitleOn:= True;
  MinMaxOn:= TRue;
  NumOn := False;
  AXPEn := New(PPen,init(PS_Solid,1,RGB(0,0,0)));
  Texton := True;
  Dif:= 0.5;
  StrLcopy(Name,nName,sizeOf(Name)-1);
  SetRect(R,0,0,1,1);
End;

Procedure TAxis.SetRange(DC:HDC;nMin,nMax : Real);
Begin
  Min := nMin;
  Max := nMax;
  If abs((Max-min)/Dif) > 100 Then
    Dif := (Max-min)/100;
End;

Procedure TAxis.SetName(nName:Pchar);
Begin
  StrLcopy(Name,nname,sizeOf(name)-1);
End;

Procedure TAxis.SetScale(ndif : Real);
Begin
  Dif := nDif;
End;


Procedure TAxis.SetRectSize(DC:HDC;nR:Trect);
Begin
  R := nR;
End;

Function TAXis.GetMinW(DC:HDC):integer;
Begin
  GetMinW := 0;
End;

Function TAXis.GetMinh(DC:HDC):integer;
Begin
  GetMinH := 0;
End;

Function TAxis.Translate(src: Real): integer;
Begin
   Translate := OffsetDest + Round(((src-OffsetSrc)*Scale));
End;

Procedure TAxis.Draw(DC:HDC);
Begin
   Abstract;
End;

Destructor TAxis.Done;
Begin
  Dispose(AxPen,done);
  Tobject.Done;
end;

Procedure THorAxis.SetRange(DC:HDC;nMin,nMax : Real);
Begin
  Min := nMin;
  Max := nMax;
  If abs((Max-min)/Dif) > 100 Then
    Dif := (Max-min)/100;
  Scale := (R.Right-R.left)/(Max-Min);
  OffsetSrc := Min;
  OffsetDest := R.Left;
End;

Procedure THorAxis.SetRectSize(DC:HDC;nR:Trect);
Begin
  R := nR;
  Scale := (R.Right-R.left)/(Max-Min);
  OffsetSrc := Min;
  OffsetDest := R.Left;
End;


Function THorAxis.GetMinh(DC:HDC) : Integer;
Var
  w : integer;
Begin
  w := AxisPt;
  If TitleOn Then
     w := w + HiWord(GetTextExtent(DC,Name,Strlen(Name)));
  If MinMaxOn or NumOn Then
     W := W + HiWord(GetTextExtent(DC,'0',1));
  GetMinH := W;
End;

Procedure THorAxis.Draw(DC:HDC);
var
 f,x,pos : integer;
  c :integer;
  num : Real;
  h : Integer;
  TextR : Trect;
  S: array[0..9] of char;
  Down : integer;
Begin
  AxPen^.Select(DC);
  h := 0;
  If MinMaxOn Then
    h := hiWord(GetTextExtent(DC,'0',1));
  If TitleOn Then
  Begin
     SetRect(TextR,R.Left,R.Top+AxisPt+h,R.Right,R.bottom);
     ExtTextOut(DC,TextR.Right-LoWord(GetTextExtent(DC,Name,StrLen(Name)))
            ,TextR.Top,eto_Clipped,@TextR,Name,StrLen(Name),Nil);
  End;
  MoveTo(DC,R.Left,R.top);
  LineTo(DC,R.right,R.top);
  MoveTo(DC,R.left,R.top);
  LineTo(DC,r.left,R.Top+AxisPt);
  MoveTo(DC,R.right,R.top);
  LineTo(DC,R.Right,R.Top+AxisPt);
  If abs((Max-min)/Dif) > 100 Then
    Dif := abs((Max-min)/100);
  c := ABS(Trunc((Max-Min)/Dif));
  If Max < Min Then
    Down := -1
  Else
    Down := 1;
  num := min;
  For f := 0 to c do
  Begin
    Pos := Translate(Min+down*(f*dif));
    x := translate(Min+Down*((f+1)*dif));
    IF  NumOn Then
    Begin
      SetRect(TextR,Pos,R.Top+Axispt,X,R.Top+Axispt+hiWord(GetTextExtent(DC,'0',1)));
      Str(Num:3:2,s);
      ExtTextOut(DC,TextR.Left,TextR.Top,eto_Clipped,@TextR,S,StrLen(S),Nil);
    End;
    num := Num + Down*dif;
    MoveTo(DC,x,R.Top);
    LineTo(DC,x,R.Top+AxisPt);
  End;
  If MinMaxOn Then
  Begin
    Str(Min:3:2,s);
    SetRect(TextR,r.Left,R.Top+Axispt,R.Left+loWord(GetTextExtent(DC,s,StrLen(s)))
       ,R.Top+Axispt+hiWord(GetTextExtent(DC,'0',1)));
      ExtTextOut(DC,TextR.Left,TextR.Top,eto_Clipped,@TextR,S,StrLen(S),Nil);
    Str(Max:3:2,s);
    SetRect(TextR,r.right-loWord(GetTextExtent(DC,s,StrLen(s))),R.Top+Axispt,R.Right
       ,R.Top+Axispt+hiWord(GetTextExtent(DC,'0',1)));
      ExtTextOut(DC,TextR.Left,TextR.Top,eto_Clipped,@TextR,S,StrLen(S),Nil);
    SetRect(TextR,R.Left,r.top+axispt,R.Right-AxisPt,R.top+h);
  End;
  AxPen^.Delete;
end;

Procedure TVerAxis.SetRange(DC:HDC;nMin,nMax : Real);
Var
  h : integer;
Begin
  Min := nMin;
  Max := nMax;
  If abs((Max-min)/Dif) > 100 Then
    Dif := (Max-min)/100;
  h := 0;
  If TitleOn Then
    h := hiWord(GetTextExtent(DC,Name,Strlen(Name)));
  Scale := ((R.top+h)-R.Bottom)/(Max-Min);
  OffsetSrc := Min;
  OffsetDest := R.Bottom;
End;

Procedure TVerAxis.SetRectSize(DC:HDC;nR:Trect);
Var
  h : integer;
Begin
  R := nR;
  h := 0;
  If TitleOn Then
    h := hiWord(GetTextExtent(DC,Name,Strlen(Name)));
  Scale := ((R.top+h)-R.Bottom)/(Max-Min);
  OffsetSrc := Min;
  OffsetDest := R.Bottom;
end;

Function TVerAxis.GetMinW(DC:HDC) : Integer;
Var
  w : integer;
  s : array[0..9] of char;
  l : integer;
Begin
  l := 0;
  w := AxisPt;
  If TitleOn Then
     w := w + loWord(GetTextExtent(DC,Name,Strlen(Name)));
  If MinMaxOn or NumOn Then
  Begin
     Str(Max:3:2,s);
     l := StrLen(S);
     Str(Min:3:2,s);
     If StrLen(s) < l THen
       Str(Max:3:2,s);
     l := AxisPt + loWord(GetTextExtent(DC,s,StrLen(s)));
  End;
  If l > w Then
    w := l;
  GetMinW := W;
End;
Function TVerAxis.GetMinh(DC:HDC) : Integer;
Var
  h : integer;
  s : array[0..9] of char;
  l : integer;
Begin
  h := 0;
  If TitleOn Then
     h := h + hiWord(GetTextExtent(DC,Name,Strlen(Name)));
  GetMinh := h;
End;

Procedure TVerAxis.Draw(DC:HDC);
var
  f,y,pos : integer;
  h,nameh : Integer;
  c :integer;
  num : Real;
  TextR : Trect;
  S: array[0..9] of char;
  Down : integer;

Begin
  AxPen^.Select(DC);
  nameh := 0;
  h := hiWord(GetTextExtent(DC,'0',1));
  If TitleOn Then
     nameh := hiWord(GetTextExtent(DC,Name,StrLen(name)));
  MoveTo(DC,R.Right,R.top+nameh);
  LineTo(DC,R.right,R.Bottom);
  MoveTo(DC,R.Right,R.top+nameh);
  LineTo(DC,R.right-axispt,R.top+nameh);
  MoveTo(DC,R.Right,R.Bottom);
  LineTo(DC,R.right-axispt,R.Bottom);
  If MinMaxOn Then
  Begin
    SetRect(TextR,R.Left,r.top+nameh,R.Right-AxisPt,R.top+h+nameh);
    Str(Max:3:2,s);
    ExtTextOut(DC,TextR.Right-LoWord(GetTextExtent(DC,s,StrLen(s))),
                  TextR.Top,eto_Clipped,@TextR,S,StrLen(S),Nil);
    SetRect(TextR,R.Left,r.Bottom,R.Right-AxisPt,R.Bottom+h);
    Str(Min:3:2,s);
    ExtTextOut(DC,TextR.Right-LoWord(GetTextExtent(DC,s,StrLen(s))),
                  TextR.top,eto_Clipped,@TextR,S,StrLen(S),Nil);
  End;
  If abs((Max-min)/Dif) > 100 Then
    Dif := abs((Max-min)/100);
  c := ABS(Trunc((Max-Min)/Dif));
  If Max < Min Then
    Down := -1
  Else
    Down := 1;
  num := min;
  For f := 0 to c do
  Begin
    Pos := Translate(Min+down*(f*dif));
    y := translate(Min+down*((f+1)*dif));
    IF  NumOn Then
    Begin
      SetRect(TextR,R.Left,Pos,R.Right-AxisPt,Pos+h);
      Str(Num:3:2,s);
      ExtTextOut(DC,TextR.Right-LoWord(GetTextExtent(DC,s,StrLen(s))),
                    TextR.Top,eto_Clipped,@TextR,S,StrLen(S),Nil);
    End;
    num := Num + down*dif;
    MoveTo(DC,r.Right,pos);
    LineTo(DC,r.Right-AxisPt,Pos);
  End;
  If TitleOn Then
  Begin
     SetRect(TextR,R.Left,R.Top,R.Right,R.top+nameh);
     ExtTextOut(DC,TextR.Left,TextR.Top,eto_Clipped,@TextR,Name,StrLen(Name),Nil);

  End;
  AxPen^.Delete;
end;
end.
