Unit Global;
Interface
Uses Objects,OMemory,OWindows,ODialogs;
Type
  Tptr = Record
    Ofs,seg: Word;
  end;

Type
   PEvalFunc = ^TEvalFunc;    {Evaluation function type}
   TEvalFunc = Function(P : Pointer;n : Integer):Real;
   FunctionToken = (Bitfunc,PackedFunc,VectFunc,ValFunc);


Const
   Err_ok    = 0;    {can run errors}
   Err_dll   = 1;
   Err_func  = 2;
   Err_cross = 3;
   Err_mut   = 4;
   Err_sel   = 5;
   Err_gen   = 6;

   FunctionCount = 4;  {number of function types supported}

   FunctionTypes : Array[0..FunctionCount-1] of pchar =
     ('BITFUNC','PACKEDFUNC','VECTFUNC','VALFUNC');


  Bits : array[0..7] of byte = (128,64,32,16,8,4,2,1);
     {bits for string compacting}

{
 11111111 = 255
 01111111 = 127
 00111111 = 63
 00011111 = 31
 00001111 = 15
 00000111 = 7
 00000011 = 3
 00000001 = 1
 00000000 = 0

 00000000 = 0
 10000000 = 128
 11000000 = 192
 11100000 = 224
 11110000 = 240
 11111000 = 248
 11111100 = 252
 11111110 = 254
 11111111 = 255
 }

  BMask1 : Array[0..8] of Byte = (255,127,63,31,15,7,3,1,0);
  BMask2 : Array[0..8] of Byte = (0,128,192,224,240,248,252,254,255);


Type
  Pinfo = ^Tinfo;
  {holds algorithm specific information}
  Tinfo = Object(TObject)
     BitLength    : Integer; {number of bits to represent a chromonsone}
     Full         : Integer; {number of bytes to encode a chromosone}
     Slop         : Integer; {remainder bits to encode a chromosone}
     BuffSize     : Integer; {Size of the buffer to hold a chromsone}
     FunctionType : FunctionToken; {function type to call}
     Minimize     : Boolean;  {Should the function minimize}
     NumPoints    : Integer; {Number of crossover points}
     CrossRate    : Real; {Crossover Rate 0.0 - 1.0}
     MutateRate   : Real; {Muataion rate 0.0 - 1.0 }
     PoolSize     : Integer; {Size ofPopulation}
     IsUserCount  : Boolean; {are the number of vectors user defined}
     UserCount    : Integer; {Number of vectors}
     IsUserLength : Boolean; {Is the crhomosone length user defined}
     GeneList     : PCollection; {Pointer to a genelist}
     CurrentBest  : Real; {Current best fitness}
     Best         : Real; {overall best}
     CurrentWorst : Real; {Current Worst fitness}
     Worst        : Real; {Overall Worst}
     CurrentAv    : Real; {Current av fitness}
     CurrentBestPos  : Integer; {Current best chromosone}{dogey}
     Bias         : Real; {BitBias of population}
     Lost         : Integer; {number of totally converged positions}
     Conv         : integer; {number of partially converged chromosones}
     mindif       : Integer; {PopSize div 20}
     Generation   : Integer; {current generation}
     CurrCount    : Integer; {current index in pop}
     CurrTrial    : Integer; {Current trial}
     Evals        : Longint; {number of evaluations carried out}
     MaxGen       : Integer; {MaxGeneration}
     MaxTRial     : Integer; {max number of trials}
     ConvPoint    : Real; {number required for convergence}
     CurrentSeed  : Integer; {current seed for system}
     SetSeed      : Boolean;
     DLLLoaded      : Boolean;
     FunctionLoaded : Boolean;
     CrossSelected  : Boolean;
     MutateSelected : Boolean;
     SelectSelected : Boolean;
     GeneralOk      : Boolean;
     Running        : Boolean;
     Constructor init;
     Procedure SetBitLength(nBits : integer);
       {sets the bitlength and modifies assoicated fields}
     Function CanRun: Integer;
       {Checks if the genetic algorithm is a ready to run state}
     Procedure SetPoolSize(nPop : Integer);
       {sets the pool size and modifies any associated fields}
     Procedure ResetRun;
       {resets any fields at the begining of a new run}
     Destructor Done; Virtual;
  end;

Const
   charsize = 8;

Procedure MakeStr(var P;L:Integer);
{This is used for the allocation of memory for a string on the heap}
{If a string has been allocated and then dealocated then reallocating it will produce a
string with garbage in it, this is fixed by setting the first char to null}
Implementation
 Procedure MakeStr(var P;L:Integer);
Begin
  GetMem(Pchar(P),L);
  Pchar(P)[0]:=#0;
End;


Constructor Tinfo.Init;
Begin
  Tobject.Init;
  BitLength    := 0;
  Full         := 0;
  Slop         := 0;
  BuffSize     := 0;
  Minimize     := True;
  NumPoints    := 1;
  CrossRate    := 0.6;
  MutateRate   := 0.001;
  PoolSize     := 100;
  IsUserCount  := False;
  UserCount    := 0;
  IsUserLength := False;
  GeneList     := nil;
  CurrentBest  := 0;
  Best         := 0;
  CurrentWorst := 0;
  Worst        := 0;
  CurrentAv    := 0;
  Bias         := 0;
  Lost         := 0;
  Conv         := 0;
  mindif       := 0;
  generation   := 1;
  CurrTRial    := 0;
  CurrCount    := 0;
  ConvPoint    := 0.95;
  MaxGen       := 150;
  CurrentSeed  := 1234;
  SetSeed      := True;
  DllLoaded      := False;
  FunctionLoaded := False;
  CrossSelected  := False;
  MutateSelected := False;
  SelectSelected := False;
  GeneralOk      := False;
  Running        := False;
End;

Function Tinfo.CanRun : integer;
Begin
  CanRun := Err_ok;
  If not GeneralOk Then
    canRun := err_gen;
  If not MutateSelected Then
    canRun := Err_mut;
  If not CrossSelected Then
    canRun := Err_cross;
  If not  SelectSelected Then
    canRun := Err_sel;
  If not  FunctionLoaded Then
    canRun := Err_func;
  If not DLLLoaded Then
    canRun := Err_dll;
End;

Procedure Tinfo.SetBitLength(nBits : Integer);
Begin
  BitLength := nBits;
  Slop := BitLength mod Charsize;
  Full := BitLength div Charsize;
  Buffsize := (BitLength div Charsize)+2;
  If Minimize Then
  Begin
    worst := 1.6e38;
    best := 0;
  end
  Else
  Begin
    worst := 0;
    best := 1.6e38;
  End;
End;

Procedure TInfo.SetPoolSize(nPop : Integer);
Begin
 PoolSize := nPop;
 MinDif := PoolSize div 20;
End;

Procedure Tinfo.ResetRun;
Begin
  If Minimize Then
  Begin
    worst := 0;
    best := 1.6e38;
  end
  Else
  Begin
    worst := 1.6e38;
    best := 0;
  End;
  If SetSeed Then
    RandSeed := CurrentSeed;
End;

Destructor Tinfo.Done;
Begin
  TObject.done;
End;
end.